@@ -30,18 +30,41 @@ import logilab.mtconverter as mtc
 import os.path as osp
 DATAPATH = osp.join(osp.split(mtc.__file__)[0], 'test')
 
-class Html2TextTC(TestCase):
+class MiscTransformsTC(TestCase):
     def test_html_to_text(self):
         data = TransformData(u'<b>yo (zou  ;)</b>', 'text/html', 'utf8')
         converted = ENGINE.convert(data, 'text/plain').decode().strip()
         self.assertEquals(converted, u'**yo (zou  ;)**')
 
+        # will trigger guess_encoding, check non-utf8 encoding
+        data = TransformData(u'<b>yo (zou  ;)</b>'.encode('latin1'), 'text/html')
+        converted = ENGINE.convert(data, 'text/plain').decode().strip()
+        self.assertEquals(converted, u'**yo (zou  ;)**')
+
+        data = TransformData(u'<p>yo <br/>zogzog </p>', 'text/html', 'utf8')
+        converted = ENGINE.convert(data, 'text/plain').decode().strip()
+        self.assertEquals(converted, u'yo\n\nzogzog')
+
     def test_xml_to_text(self):
         data = TransformData(u'<root><b>yo (zou  ;)</b>a<tag/>b<root>', 'application/xml', 'utf8')
         converted = ENGINE.convert(data, 'text/plain').decode().strip()
         self.assertEquals(converted, u'yo (zou  ;) a b')
 
-class Odt2TextTC(TestCase):
+
+    def test_pgpsignature_to_text(self):
+        _data = u"""-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.9 (GNU/Linux)
+
+iEYEARECAAYFAkxX5p8ACgkQkjcInxztrI64QQCggKA+PmbLYnGNtBB3Lb3pO3P8
+r2MAoIO1DSsuM23SzgmqubGJEZuSRWhR
+=GDDk
+-----END PGP SIGNATURE-----
+"""
+        data = TransformData(_data, 'application/pgp-signature')
+        converted = ENGINE.convert(data, 'text/plain').decode()
+        self.assertTextEquals(converted, _data)
+
+
     def test_odt_to_text(self):
         data = TransformData(open(osp.join(DATAPATH, 'hello.odt')),
                              'application/vnd.oasis.opendocument.text', 'utf8')
