@@ -24,18 +24,18 @@ Copyright (C) 2004-2008 Aaron Swartz. GNU GPL 3.
 Copyright (C) 2008 Logilab S.A.
 """
 
-__version__ = "2.3"
+__version__ = "2.38"
 __author__ = "Aaron Swartz (me@aaronsw.com)"
 __copyright__ = "(C) 2004-2008 Aaron Swartz. GNU GPL 3."
-__contributors__ = ["Martin 'Joey' Schulze", "Ricardo Reyes"]
+__contributors__ = ["Martin 'Joey' Schulze", "Ricardo Reyes", "Kevin Jay North"]
 
 # TODO:
 #   Support decoded entities with unifiable.
-#   Relative URL resolution
 
 if not hasattr(__builtins__, 'True'): True, False = 1, 0
 import re, sys, urllib, htmlentitydefs, codecs, StringIO, types
 import sgmllib
+import urlparse
 sgmllib.charref = re.compile('&#([xX]?[0-9a-fA-F]+)[^0-9a-fA-F]')
 
 try: from textwrap import wrap
@@ -65,13 +65,13 @@ def name2cp(k):
         if k.startswith("&#") and k.endswith(";"): return int(k[2:-1]) # not in latin-1
         return ord(codecs.latin_1_decode(k)[0])
 
-unifiable = {'rsquo':"'", 'lsquo':"'", 'rdquo':'"', 'ldquo':'"', 
+unifiable = {'rsquo':"'", 'lsquo':"'", 'rdquo':'"', 'ldquo':'"',
 'copy':'(C)', 'mdash':'--', 'nbsp':' ', 'rarr':'->', 'larr':'<-', 'middot':'*',
 'ndash':'-', 'oelig':'oe', 'aelig':'ae',
-'agrave':'a', 'aacute':'a', 'acirc':'a', 'atilde':'a', 'auml':'a', 'aring':'a', 
-'egrave':'e', 'eacute':'e', 'ecirc':'e', 'euml':'e', 
+'agrave':'a', 'aacute':'a', 'acirc':'a', 'atilde':'a', 'auml':'a', 'aring':'a',
+'egrave':'e', 'eacute':'e', 'ecirc':'e', 'euml':'e',
 'igrave':'i', 'iacute':'i', 'icirc':'i', 'iuml':'i',
-'ograve':'o', 'oacute':'o', 'ocirc':'o', 'otilde':'o', 'ouml':'o', 
+'ograve':'o', 'oacute':'o', 'ocirc':'o', 'otilde':'o', 'ouml':'o',
 'ugrave':'u', 'uacute':'u', 'ucirc':'u', 'uuml':'u'}
 
 unifiable_n = {}
@@ -84,7 +84,7 @@ def charref(name):
         c = int(name[1:], 16)
     else:
         c = int(name)
-    
+
     if not UNICODE_SNOB and c in unifiable_n.keys():
         return unifiable_n[c]
     else:
@@ -100,14 +100,14 @@ def entityref(c):
 
 def replaceEntities(s):
     s = s.group(1)
-    if s[0] == "#": 
+    if s[0] == "#":
         return charref(s[1:])
     else: return entityref(s)
 
 r_unescape = re.compile(r"&(#?[xX]?(?:[0-9a-fA-F]+|\w{1,8}));")
 def unescape(s):
     return r_unescape.sub(replaceEntities, s)
-    
+
 def fixattrs(attrs):
     # Fix bug in sgmllib.py
     if not attrs: return attrs
@@ -129,8 +129,8 @@ def optwrap(text):
     """Wrap all paragraphs in the provided text."""
     if not BODY_WIDTH:
         return text
-    
-    assert wrap # Requires Python 2.3.
+
+    assert wrap, "Requires Python 2.3."
     result = ''
     newlines = 0
     for para in text.split("\n"):
@@ -158,9 +158,10 @@ def hn(tag):
         except ValueError: return 0
 
 class _html2text(sgmllib.SGMLParser):
-    def __init__(self, out=sys.stdout.write):
+
+    def __init__(self, out=None, baseurl='', encoding='utf8'):
         sgmllib.SGMLParser.__init__(self)
-        
+
         if out is None: self.out = self.outtextf
         else: self.out = out
         self.outtext = []
@@ -177,44 +178,50 @@ class _html2text(sgmllib.SGMLParser):
         self.pre = 0
         self.startpre = 0
         self.lastWasNL = 0
-    
-    def outtextf(self, s): 
-        if type(s) is str: s = codecs.utf_8_decode(s)[0]
-        self.outtext.append(s)
-    
+        self.abbr_title = None # current abbreviation definition
+        self.abbr_data = None # last inner HTML (for abbr being defined)
+        self.abbr_list = {} # stack of abbreviations to write later
+        self.baseurl = baseurl
+        self._encoding = encoding
+
+    def outtextf(self, s):
+        if isinstance(s, str):
+            s = unicode(s, self._encoding)
+        self.outtext.append( s )
+
     def close(self):
         sgmllib.SGMLParser.close(self)
-        
+
         self.pbr()
         self.o('', 0, 'end')
-        
-        return u''.join(self.outtext)
-        
+
+        return ''.join(self.outtext)
+
     def handle_charref(self, c):
         self.o(charref(c))
 
     def handle_entityref(self, c):
         self.o(entityref(c))
-            
+
     def unknown_starttag(self, tag, attrs):
         self.handle_tag(tag, attrs, 1)
-    
+
     def unknown_endtag(self, tag):
         self.handle_tag(tag, None, 0)
-        
+
     def previousIndex(self, attrs):
         """ returns the index of certain set of attributes (of a link) in the
             self.a list
- 
+
             If the set of attributes is not found, returns None
         """
         if not attrs.has_key('href'): return None
-        
+
         i = -1
         for a in self.a:
             i += 1
             match = 0
-            
+
             if a.has_key('href') and a['href'] == attrs['href']:
                 if a.has_key('title') or attrs.has_key('title'):
                         if (a.has_key('title') and attrs.has_key('title') and
@@ -227,13 +234,13 @@ class _html2text(sgmllib.SGMLParser):
 
     def handle_tag(self, tag, attrs, start):
         attrs = fixattrs(attrs)
-    
+
         if hn(tag):
             self.p()
             if start: self.o(hn(tag)*"#" + ' ')
 
         if tag in ['p', 'div']: self.p()
-        
+
         if tag == "br" and start: self.o("  \n")
 
         if tag == "hr" and start:
@@ -241,31 +248,46 @@ class _html2text(sgmllib.SGMLParser):
             self.o("* * *")
             self.p()
 
-        if tag in ["head", "style", 'script']: 
+        if tag in ["head", "style", 'script']:
             if start: self.quiet += 1
             else: self.quiet -= 1
 
         if tag in ["body"]:
             self.quiet = 0 # sites like 9rules.com never close <head>
-        
+
         if tag == "blockquote":
-            if start: 
+            if start:
                 self.p(); self.o('> ', 0, 1); self.start = 1
                 self.blockquote += 1
             else:
                 self.blockquote -= 1
                 self.p()
-        
+
         if tag in ['em', 'i', 'u']: self.o("_")
         if tag in ['strong', 'b']: self.o("**")
         if tag == "code" and not self.pre: self.o('`') #TODO: `` `this` ``
-        
+        if tag == "abbr":
+            if start:
+                attrsD = {}
+                for (x, y) in attrs: attrsD[x] = y
+                attrs = attrsD
+
+                self.abbr_title = None
+                self.abbr_data = ''
+                if attrs.has_key('title'):
+                    self.abbr_title = attrs['title']
+            else:
+                if self.abbr_title != None:
+                    self.abbr_list[self.abbr_data] = self.abbr_title
+                    self.abbr_title = None
+                self.abbr_data = ''
+
         if tag == "a":
             if start:
                 attrsD = {}
                 for (x, y) in attrs: attrsD[x] = y
                 attrs = attrsD
-                if attrs.has_key('href') and not (SKIP_INTERNAL_LINKS and attrs['href'].startswith('#')): 
+                if attrs.has_key('href') and not (SKIP_INTERNAL_LINKS and attrs['href'].startswith('#')):
                     self.astack.append(attrs)
                     self.o("[")
                 else:
@@ -283,7 +305,7 @@ class _html2text(sgmllib.SGMLParser):
                             a['outcount'] = self.outcount
                             self.a.append(a)
                         self.o("][" + `a['count']` + "]")
-        
+
         if tag == "img" and start:
             attrsD = {}
             for (x, y) in attrs: attrsD[x] = y
@@ -302,20 +324,20 @@ class _html2text(sgmllib.SGMLParser):
                 self.o("![")
                 self.o(alt)
                 self.o("]["+`attrs['count']`+"]")
-        
+
         if tag == 'dl' and start: self.p()
         if tag == 'dt' and not start: self.pbr()
         if tag == 'dd' and start: self.o('    ')
         if tag == 'dd' and not start: self.pbr()
-        
+
         if tag in ["ol", "ul"]:
             if start:
                 self.list.append({'name':tag, 'num':0})
             else:
                 if self.list: self.list.pop()
-            
+
             self.p()
-        
+
         if tag == 'li':
             if start:
                 self.pbr()
@@ -329,10 +351,10 @@ class _html2text(sgmllib.SGMLParser):
                 self.start = 1
             else:
                 self.pbr()
-        
+
         if tag in ["table", "tr"] and start: self.p()
         if tag == 'td': self.pbr()
-        
+
         if tag == "pre":
             if start:
                 self.startpre = 1
@@ -340,32 +362,34 @@ class _html2text(sgmllib.SGMLParser):
             else:
                 self.pre = 0
             self.p()
-            
+
     def pbr(self):
         if self.p_p == 0: self.p_p = 1
 
     def p(self): self.p_p = 2
-    
+
     def o(self, data, puredata=0, force=0):
-        if not self.quiet: 
+        if self.abbr_data is not None: self.abbr_data += data
+
+        if not self.quiet:
             if puredata and not self.pre:
                 data = re.sub('\s+', ' ', data)
                 if data and data[0] == ' ':
                     self.space = 1
                     data = data[1:]
             if not data and not force: return
-            
+
             if self.startpre:
                 #self.out(" :") #TODO: not output when already one there
                 self.startpre = 0
-            
+
             bq = (">" * self.blockquote)
             if not (force and data and data[0] == ">") and self.blockquote: bq += " "
-            
+
             if self.pre:
                 bq += "    "
                 data = data.replace("\n", "\n"+bq)
-            
+
             if self.start:
                 self.space = 0
                 self.p_p = 0
@@ -381,7 +405,7 @@ class _html2text(sgmllib.SGMLParser):
             if self.p_p:
                 self.out(('\n'+bq)*self.p_p)
                 self.space = 0
-                
+
             if self.space:
                 if not self.lastWasNL: self.out(' ')
                 self.space = 0
@@ -392,7 +416,7 @@ class _html2text(sgmllib.SGMLParser):
                 newa = []
                 for link in self.a:
                     if self.outcount > link['outcount']:
-                        self.out("   ["+`link['count']`+"]: " + link['href']) #TODO: base href
+                        self.out("   ["+`link['count']`+"]: " + urlparse.urljoin(self.baseurl, link['href']))
                         if link.has_key('title'): self.out(" ("+link['title']+")")
                         self.out("\n")
                     else:
@@ -402,27 +426,32 @@ class _html2text(sgmllib.SGMLParser):
 
                 self.a = newa
 
+            if self.abbr_list and force == "end":
+                for abbr, definition in self.abbr_list.items():
+                    self.out("  *[" + abbr + "]: " + definition + "\n")
+
             self.p_p = 0
             self.out(data)
             self.lastWasNL = data and data[-1] == '\n'
             self.outcount += 1
 
     def handle_data(self, data):
+        if r'\/script>' in data: self.quiet -= 1
         self.o(data, 1)
-    
+
     def unknown_decl(self, data): pass
 
-def wrapwrite(text):
-    sys.stdout.write(text.encode('utf8'))
+def wrapwrite(text): sys.stdout.write(text.encode('utf8'))
 
-def html2text_file(html, out=wrapwrite):
-    h = _html2text(out)
+def html2text_file(html, out=wrapwrite, baseurl='', encoding='utf8'):
+    h = _html2text(out, baseurl, encoding=encoding)
     h.feed(html)
     h.feed("")
     return h.close()
 
-def html2text(html):
-    return optwrap(html2text_file(html, None))
+def html2text(html, baseurl='', encoding='utf8'):
+    return optwrap(html2text_file(html.replace('/>', '>'), None,
+                                  baseurl, encoding=encoding))
 
 
 ## mtconverter's specific code ################################################
@@ -438,4 +467,4 @@ class html_to_formatted_text(Transform):
 
 
     def _convert(self, trdata):
-        return html2text(trdata.data)
+        return html2text(trdata.data, encoding=trdata.encoding)
